﻿#ifndef AlivcLivePlayr_Main_Dialog_h
#define AlivcLivePlayr_Main_Dialog_h

#include "AlivcAudioRender.h"
#include "AlivcDownloaderDlg.h"
#include "AlivcNetworkManager.h"
#include "AlivcPlayerMpsDlg.h"
#include "AlivcPlayerPlayAuthDlg.h"
#include "AlivcPlayerPlayConfigDlg.h"
#include "AlivcPlayerProgress.h"
#include "AlivcPlayerStsDlg.h"
#include "AlivcVideoRender.h"
#include "sdk_headers/AliPlayer.h"
#include <QCheckBox>
#include <QComboBox>
#include <QDialog>
#include <QLabel>
#include <QProgressBar>
#include <QPushButton>
#include <QRadioButton>
#include <QSlider>
#include <QTextEdit>
#include <mutex>

using namespace std;
using namespace alivc_player;

class AlivcLivePlayerMainDlg : public QDialog, public IAVPListener
{
  Q_OBJECT

public:
  enum AlivcPlaySouceType
  {
    AlivcPlaySourceUrlSource,
    AlivcPlaySourceVidSource,
  };

  enum AlivcVidSourceType {
      AlivcVidTypeSts,
      AlivcVidTypePlayAuth,
      AlivcVidTypeMps
  };

  AlivcLivePlayerMainDlg(QWidget *parent = Q_NULLPTR);
  ~AlivcLivePlayerMainDlg();

  static void logCallback(void *userData, int prio, char *buf);
  static void LogToFile(const char *fmt, ...);

public:
  /**
  @brief 播放器事件回调
  @param player 播放器player指针
  @param eventType 播放器事件类型，@see AVPEventType
  */
  virtual void onPlayerEvent(AliPlayer *player, AVPEventType eventType) override;

  /**
  @brief 播放器事件回调
  @param player 播放器player指针
  @param eventWithString 播放器事件类型
  @param description 播放器事件说明
  */
  virtual void onPlayerEvent(AliPlayer *player, AVPEventWithString eventWithString, const char *description) override;

  /**
  @brief 错误代理回调
  @param player 播放器player指针
  @param errorModel 播放器错误描述，参考AliVcPlayerErrorModel
  */
  virtual void onError(AliPlayer *player, AVPErrorModel *errorModel) override;

  /**
  @brief 视频大小变化回调
  @param player 播放器player指针
  @param width 视频宽度
  @param height 视频高度
  @param rotation 视频旋转角度
  */
  virtual void onVideoSizeChanged(AliPlayer *player, int width, int height, int rotation) override;

  /**
  @brief 视频当前播放位置回调
  @param player 播放器player指针
  @param position 视频当前播放位置
  */
  virtual void onCurrentPositionUpdate(AliPlayer *player, int64_t position) override;

  /**
  @brief 视频缓存位置回调
  @param player 播放器player指针
  @param position 视频当前缓存位置
  */
  virtual void onBufferedPositionUpdate(AliPlayer *player, int64_t position) override;

  /**
  @brief 视频缓冲进度回调
  @param player 播放器player指针
  @param progress 缓存进度0-100
  */
  virtual void onLoadingProgress(AliPlayer *player, float progress) override;

  /**
  @brief 获取track信息回调
  @param player 播放器player指针
  @param info track流信息数组 参考AVPTrackInfo
  */
  virtual void onTrackReady(AliPlayer *player, AVPTrackInfo* info[],int count) override;

  /**
  @brief track切换完成回调
  @param player 播放器player指针
  @param info 切换后的信息 参考AVPTrackInfo
  */
  virtual void onTrackChanged(AliPlayer *player, AVPTrackInfo *info) override;

  virtual void onSubtitleExtAdded(AliPlayer *player, int64_t trackIndex, const char *URL) override;

  virtual void onSubtitleShow(AliPlayer *player, int64_t trackIndex, int64_t subtitleId, const char *subtitle) override;

  virtual void onSubtitleHide(AliPlayer *player, int64_t trackIndex, int64_t subtitleId) override;

  /**
  @brief 获取缩略图成功回调
  @param positionMs 指定的缩略图位置
  @param fromPos 此缩略图的开始位置
  @param toPos 此缩略图的结束位置
  @param image 缩图略图像指针,对于mac是NSImage，iOS平台是UIImage指针
  */
  virtual void onGetThumbnailSuc(int64_t positionMs, int64_t fromPos, int64_t toPos, void *image) override;

  /**
  @brief 获取缩略图失败回调
  @param positionMs 指定的缩略图位置
  */
  virtual void onGetThumbnailFailed(int64_t positionMs) override;

  /**
  @brief 播放器状态改变回调
  @param player 播放器player指针
  @param oldStatus 老的播放器状态 参考AVPStatus
  @param newStatus 新的播放器状态 参考AVPStatus
  */
  virtual void onPlayerStatusChanged(AliPlayer *player, AVPStatus oldStatus, AVPStatus newStatus) override;

  virtual void onSnapshotImageBuffer(AliPlayer* player, int width, int height, unsigned char* pARGBBuffer) override;

private:
  void InitLayout();
  bool InitEvent();
  void InitLog();
  void InitApsaraPlayer();
  void NotifySDKError(int error);
  void ChangeFullScreenState();
  void SeekTo(int pos);
  void ChangeVolume(int volume);
  void UpdateStatus();
  void UpdatePlaySource();
  void ResetUI();
  static bool OnRenderingFrame(void *userData, AVPFrameInfo *frameInfo);

signals:
  void SgnUpdateUI();
  void SgnErrorUI(QString errorMsg);
  void SgnPreparedUI();
  void SgnProgressUI(qint64 pos);
  void SgnUpdateBuffer(int pos);
  void SgnUpdateSubtitle(QString text,bool show);
  void SgnAddExtSubtitle(qint64 trackIndex);
  void SgnCompletionUI();
  void SgnStopUI();
  void SgnPauseUI();
  void SgnResumeUI();
  void SgnStatusUI();
  void SgnFinish(QString msg);
  void SgnUpdateDefinition();
  void SgnLoadingChanged(bool isLoading);
  void SgnFirstRendered(quint64 tickCount);
  void SgnSeekEnd(quint64 tickCount);

protected:
  bool eventFilter(QObject *widget, QEvent *event) override;
  void keyPressEvent(QKeyEvent *event) override;
  void closeEvent(QCloseEvent *event) override;

  private slots:
  void OnUpdateUI();
  void OnPrearedUI();
  void OnProgressUI(qint64 pos);
  void OnUpdateBuffer(int pos);
  void OnProgressValueChange(float pos);
  void OnCompletionUI();
  void OnStopUI();
  void OnPauseUI();
  void OnResumeUI();
  void OnStatusUI();
  void onUpdateSuttile(QString text, bool show);
  void onExtSuttileAdded(qint64 trackIndex);
  void OnErrorUI(QString errorMsg);
  void OnPlayButtonClicked(bool checked);
  void OnStopButtonClicked(bool checked);
  void OnPrepareButtonClicked(bool checked);
  void OnPauseButtonClicked(bool checked);
  void OnPathButtonClicked(bool checked);
  void OnConfigButtonClicked(bool checked);
  void OnBtnCacheConfigClicked(bool checked);
  void OnBtnPlayConfigClicked(bool checked);
  void OnBtnDownloadDlgClicked(bool checked);
  void OnPlayListButtonClicked(bool checked);
  void OnResumeButtonClicked(bool checked);
  void OnFullScreenButtonClicked(bool checked);
  void OnBtnSnapshotClicked(bool checked);
  void OnBtnAddExtSubtitleClicked(bool checked);
  void OnMuteStateChanged(int state);
  void OnAutoPlayStateChanged(int state);
  void OnLoopStateChanged(int state);
  void OnAppRenderStateChanged(int state);
  void OnRightRenderStateChanged(int state);
  void OnAutoABRStateChanged(int state);
  void OnVolumeChange(int position);
  void OnFinish(QString msg);
  void OnPlayByUrlButtonClicked(bool checked);
  void OnPlayByVidButtonClicked(bool checked);
  void OnStsButtonClicked();
  void OnPlayAuthButtonClicked();
  void OnMpsButtonClicked();
  void OnEncryptDataButtonClicked();
  void OnResetPlayerButtonClicked();
  void OnNewInstanceButtonClicked();
  void OnScalingModeChanged(int index);
  void OnMirrorModeChanged(int index);
  void OnRotateModeChanged(int index);
  void OnRateChanged(int index);
  void OnUpdateDefinition();
  void OnDefinitionChanged(int index);
  void OnAudioChanged(int index);
  void OnSubtitleChanged(int index);
  void OnExtSubtitleChanged(int index);
  void OnLoadingChanged(bool isLoading);
  void OnPlayWithUrl(QString url);
  void OnFirstRendered(quint64 tickCount);
  void OnSeekEnd(quint64 tickCount);

private:
  QString     mPlayURL;
  QWidget     *mPlayerViewPtr{ NULL };
  QWidget     *mAppRenderViewPtr{ NULL };
  QLabel      *mStatisticsPtr{ NULL };
  QLabel      *mStatusPtr{ NULL };
  QLabel      *mLabelLoading{ NULL };
  QLabel      *mSubtitleLabelPtr{ NULL };
  QLabel      *mPlaySouceLabel{ NULL };
  QLineEdit   *mPlayerSourceEdit{ NULL };
  QWidget     *mPlaySourceWraper{ NULL };
  QWidget     *mUrlEditWraper{ NULL };
  QWidget     *mButtonWraper{ NULL };
  QWidget     *mStatsWraper{ NULL };
  QRadioButton* mPlayByUrl{ NULL };
  QRadioButton* mPlayByVid{ NULL };
  QPushButton *mBtnSts{NULL};
  QPushButton *mBtnPlayAuth{ NULL };
  QPushButton *mBtnMps{ NULL };
  QPushButton *mBtnEncryptData{ NULL };
  QPushButton *mBtnResetPlayer{ NULL };
  QPushButton *mBtnNewInstance{ NULL };
  QPushButton *mSourceChosenPtr{ NULL };
  QPushButton *mConfigButton{ NULL };
  QPushButton *mBtnCacheConfig = nullptr;
  QPushButton *mBtnPlayConfig = nullptr;
  QPushButton *mBtnDownloadDlg = nullptr;
  QPushButton *mPrepareButtonPtr{ NULL };
  QPushButton *mPlayerButtonPtr{ NULL };
  QPushButton *mStopButtonPtr{ NULL };
  QPushButton *mPauseButtonPtr{ NULL };
  QPushButton *mResumtButtonPtr{ NULL };
  QPushButton *mFullScreenButtonPtr{ NULL };
  QPushButton *mBtnSnapshot{ NULL };
  QCheckBox   *mMuteCheckbox{ NULL };
  QCheckBox   *mAutoPlayCheckbox{ NULL };
  QCheckBox   *mLoopCheckbox{ NULL };
  QCheckBox   *mAccurateSeekCheckbox{ NULL };
  QCheckBox   *mAutoABRCheckbox{ NULL };
  QSlider     *mVolumeSlider{ NULL };
  QComboBox   *mScalingModeCombox{ nullptr };
  QComboBox   *mMirrorModeCombox{ nullptr };
  QComboBox   *mRotateModeCombox{ nullptr };
  QComboBox   *mRateCombox{ nullptr };
  QComboBox   *mDefinitionCombox{ nullptr };
  QComboBox   *mAudioTrackCombox{ nullptr };
  QComboBox   *mSubtitleTrackCombox{ nullptr };
  QPushButton *mBtnAddExtSubtitle = nullptr;
  QComboBox *mExtSubtitleCombox = nullptr;
  bool         mFullScreen{ false };
  QLabel      *mProgressLabel{ NULL };
  QCheckBox *mCheckboxRightRender{nullptr};
  QCheckBox *mCheckboxAppRender{nullptr};
  QLabel *mLabelAppAudioDevice{nullptr};
  QComboBox *mComboAppAudioDevice{nullptr};
  QLabel *mVolumeLabel{NULL};
  AlivcPlayerProgress* mPlayerProgress{ NULL };

  QWidget     *mPrgressWraper{ NULL };
  AliPlayer* mPlayerPtr{ nullptr };
  AVPStatus mPlayerStatus{ AVPStatusIdle };
  AlivcPlaySouceType mPlaySourceType{ AlivcPlaySourceUrlSource };
  AlivcStsInfo mPlayStsInfo;
  AlivcPlayAuthInfo mPlayAuthInfo;
  AlivcMpsInfo mPlayMpsInfo;
  std::mutex mTrackMutex;
  std::map<int, std::string> mTrackMap;
  AlivcVidSourceType mVidType = AlivcVidTypeSts;
  QString mSnapshotFolder;
  AlivcPlayConfig mPlayConfig;
  int64_t mLastExtSubtitleIndex = -1;
  AlivcDownloaderDlg *mDownloaderDlg = nullptr;
  static QFile *mLogFile;
  static std::mutex mLogMutex;
  AVPCacheConfig mCacheConfig;
  AlivcVideoRender *mVideoRender = nullptr;
  AlivcAudioRender *mAudioRender = nullptr;
  ULONGLONG mLaunchTime = 0;
  ULONGLONG mSeekTime = 0;
};

#endif // AlivcLivePlayr_Main_Dialog_h
