﻿//
//  AVPSource.h
//
//  Created by shiping.csp on 2018/11/16.
//  Copyright (c) 2018 com.alibaba.AliyunPlayer. All rights reserved.
//

#ifndef AVPSOURCE_H
#define AVPSOURCE_H

#include <stdint.h>
#include <stdlib.h>

#include "AVPDef.h"

namespace alivc_player {

    class ALIVC_EXTERN AVPSource {

    public:
        AVPSource();

        virtual ~AVPSource();

    public:
        void setCoverURL(const char *url);
        void setTitle(const char *title);
        void setQuality(const char *quality);
        char *getCoverURL() const;
        char *getTitle() const;
        char *getQuality() const;

    protected:
        /**
         @brief 封面地址
         */
        /****
         @brief The URL of the album cover.
         */
        char *mCoverURL;

        /**
         @brief 视频标题
         */
        /****
         @brief The title of the video.
         */
        char *mTitle;

        /**
         @brief 期望播放的清晰度
         */
        /****
         @brief The preferred definition for playback.
         */
        char *mQuality;
    };

    class ALIVC_EXTERN AVPUrlSource : public AVPSource {

    public:
        AVPUrlSource();

        ~AVPUrlSource();

    public:
        void setUrl(const char *url);
        char *getUrl() const;

    private:
        char *mUrl;
    };


    class ALIVC_EXTERN AVPVidStsSource : public AVPSource {
    public:
        AVPVidStsSource();

        ~AVPVidStsSource();

    public:
        /**
         @brief 用vid和sts来播放视频，临时AccessKeyId、AccessKeySecret和SecurityToken：开启RAM授权，并通过STS授权系统提供的OpenAPI或SDK获取的AccessKeyId、AccessKeySecret和SecurityToken，用于播放和下载请求参数明细：https://help.aliyun.com/document_detail/28788.html
         @param vid 视频播放的vid标识
         @param accessKeyId 用户sts的accessKey ID
         @param accessKeySecret 用户sts的accessKey secret
         @param securityToken 用户sts的token信息
         @param region 用户sts的region信息
         */
        /****
         @brief Play by VID and STS. Temporary AccessKey ID, AccessKey Secret, and token: Activate RAM, and use the API or SDK provided by STS to retrieve the AccessKey ID, AccessKey Secret, and token. Details about parameters in playback and download requests: https://help.aliyun.com/document_detail/28788.html
         @param vid The VID of the video.
         @param accessKeyId The AccessKey ID of the user.
         @param accessKeySecret The AccessKey Secret of the user.
         @param securityToken The token information of the user.
         @param region The region information of the user.
         */
        void initWithVid(const char *vid, const char *accessKeyId, const char *accessKeySecret, const char *securityToken,
                         const char *region, const char *playConfig);

        char *getVid() const;
        char *getAccessKeyId() const;
        char *getAccessKeySecret() const;
        char *getSecurityToken() const;
        char *getRegion() const;
        char *getPlayConfig() const;

    private:
        char *mVid;
        char *mAccessKeyId;
        char *mAccessKeySecret;
        char *mSecurityToken;
        char *mRegion;
        char *mPlayConfig;
    };

    class ALIVC_EXTERN AVPVidAuthSource : public AVPSource {
    public:
        AVPVidAuthSource();

        ~AVPVidAuthSource();

    public:
        void initWithVid(const char *vid, const char *playAuth, const char *region, const char *playConfig);

        char *getVid() const;
        char *getPlayAuth() const;
        char *getRegion() const;
        char *getPlayConfig() const;

    private:
        char *mVid;
        char *mPlayAuth;
        char *mRegion;
        char *mPlayConfig;
    };

    class ALIVC_EXTERN AVPVidMpsSource : public AVPSource {

    public:
        AVPVidMpsSource();

        ~AVPVidMpsSource();

    public:
        /**
         @brief 用vid和MPS信息来播放视频，视频转码服务用户使用播放方式。部分参数参考:https://help.aliyun.com/document_detail/53522.html
         @param vid 视频播放的vid标识
         @param accId 用户sts的accessKey ID
         @param accSecret 用户sts的accessKey secret
         @param stsToken 用户sts的token信息
         @param authInfo 用户authInfo信息
         @param region 用户region信息
         @param playDomain 用户playDomain信息
         @param mtsHlsUriToken 用户mtsHlsUriToken信息
         */
        /****
         @brief Play by VID and MPS. This playback method is used by video transcoding service users. Reference for some of the parameters: https://help.aliyun.com/document_detail/53522.html
         @param vid The VID of the video.
         @param accId The AccessKey ID of the user.
         @param accSecret The AccessKey Secret of the user.
         @param stsToken The token information of the user.
         @param authInfo The authInfo of the user.
         @param region The region information of the user.
         @param playDomain The playDomain information of the user.
         @param mtsHlsUriToken The mtsHlsUriToken information of the user.
         */
        void initWithVid(const char *vid, const char *accId, const char *accSecret, const char *stsToken, const char *authInfo,
                         const char *region, const char *playDomain, const char *mtsHlsUriToken);

        char *getVid() const;
        char *getAccId() const;
        char *getAccSecret() const;
        char *getStsToken() const;
        char *getAuthInfo() const;
        char *getRegion() const;
        char *getPlayDomain() const;
        char *getMtsHlsUriToken() const;

    private:
        char *mVid;
        char *mAccId;
        char *mAccSecret;
        char *mStsToken;
        char *mAuthInfo;
        char *mRegion;
        char *mPlayDomain;
        char *mMtsHlsUriToken;
    };
};// namespace alivc_player

#endif /* AVPSOURCE_H */
