﻿//
//  AliMediaDownloader.h
//
//  Created by kongjian.kongjian on 2020/06/04.
//

#ifndef AliMediaDownloader_h
#define AliMediaDownloader_h

#include "AVDConfig.h"
#include "AVPDef.h"
#include "AVPMediaInfo.h"
#include "AVPSource.h"

namespace alivc_player {

    class AliMediaDownloader;

    class IAVDListener {
    public:
        /**
         @brief 下载准备完成事件回调
         @param downloader 下载downloader指针
         @param mediaInfo 媒体信息，@see AVPMediaInfo
         */
        /****
         @brief Download preparation completion callback.
         @param downloader Downloader pointer.
         @param mediaInfo media information. @see AVPMediaInfo
         */
        virtual void onPrepared(AliMediaDownloader *downloader, AVPMediaInfo *mediaInfo) = 0;

        /**
         @brief 错误代理回调
         @param downloader 下载downloader指针
         @param errorModel 播放器错误描述，参考AliVcPlayerErrorModel
         */
        /****
         @brief Proxy error callback.
         @param downloader Downloader pointer.
         @param errorModel Player error description. See AliVcPlayerErrorModel
         */
        virtual void onError(AliMediaDownloader *downloader, AVPErrorModel *errorModel) = 0;

        /**
         @brief 下载进度回调
         @param downloader 下载downloader指针
         @param percent 下载进度 0-100
         */
        /****
         @brief Download progress callback.
         @param downloader Downloader pointer.
         @param percent Download progress: 0 to 100.
         */
        virtual void onDownloadingProgress(AliMediaDownloader *downloader, int percent) = 0;

        /**
         @brief 下载文件的处理进度回调
         @param downloader 下载downloader指针
         @param percent 下载进度 0-100
         */
        /****
         @brief Downloaded file processing progress callback.
         @param downloader Downloader pointer.
         @param percent Processing progress: 0 to 100.
         */
        virtual void onProcessingProgress(AliMediaDownloader *downloader, int percent) = 0;

        /**
         @brief 下载完成回调
         @param downloader 下载downloader指针
         */
        /****
         @brief Download completion callback.
         @param downloader Downloader pointer.
         */
        virtual void onCompletion(AliMediaDownloader *downloader) = 0;
    };

    class ALIVC_EXTERN AliMediaDownloader {
    public:
        static AliMediaDownloader *CreateMediaDownloader();
        static void DestroyDownloader(AliMediaDownloader *downloader);

        virtual ~AliMediaDownloader() = default;

        /**
         @brief 删除下载文件
         @param saveDir 文件保存路径
         @param vid     vid
         @param format  格式
         @param index   vid对应的下载索引
         */
        /****
         @brief Delete a downloaded file.
         @param saveDir The path where the file is saved.
         @param vid     VID.
         @param format  Format.
         @param index   The download index corresponds to the VID.
         */
        static int deleteFile(const char *saveDir, const char *vid, const char *format, int index);

        /**
         @brief 使用AVPVidStsSource准备播放
         @param source vid sts的播放方式
         */
        /****
         @brief Use AVPVidStsSource to prepare for playback.
         @param source vid Play the media through VidSts.
         */
        virtual void prepareWithVid(AVPVidStsSource *source) = 0;

        /**
         @brief 鉴权过期，更新AVPVidStsSource信息，
         @param source vid sts的信息
         */
        /****
         @brief Update the AVPVidStsSource information when the authentication expires.
         @param source vid STS information.
         */
        virtual void updateWithVid(AVPVidStsSource *source) = 0;

        /**
         @brief 使用AVPVidAuthSource准备播放
         @param source vid playAuth的播放方式
         */
        /****
         @brief Use AVPVidAuthSource to prepare for playback.
         @param source vid Play the media through playAuth.
         */
        virtual void prepareWithPlayAuth(AVPVidAuthSource *source) = 0;

        /**
         @brief 鉴权过期，更新AVPVidAuthSource信息，
         @param source vid auth的信息
         */
        /****
         @brief Update the AVPVidAuthSource information when the authentication expires.
         @param source vid Authentication information.
         */
        virtual void updateWithPlayAuth(AVPVidAuthSource *source) = 0;

        /**
         @brief 设置下载的保存路径
         @param dir 保存文件夹
         */
        /****
         @brief Set a path to save the downloaded file.
         @param dir The specified path.
         */
        virtual void setSaveDirectory(const char *dir) = 0;

        /**
         @brief 开始下载
         */
        /****
         @brief Start downloading.
         */
        virtual void start() = 0;

        /**
         @brief 停止下载
         */
        /****
         @brief Pause downloading.
         */
        virtual void stop() = 0;

        /**
         @brief 删除下载文件
         */
        /****
         @brief Delete the downloaded file.
         */
        virtual void deleteDownloadedFile() = 0;

        /**
         @brief 获取下载config
         */
        /****
         @brief Query download config.
         */
        virtual AVDConfig *getConfig() = 0;

        /**
         @brief 设置下载config
         */
        /****
         @brief Modify download config.
         */
        virtual void setConfig(AVDConfig *config) = 0;

        /**
         @brief 设置下载的trackIndex
         @param trackIndex 从prepare回调中可以获取所有index
         */
        /****
         @brief Specify the indexes of the tracks to be downloaded.
         @param trackIndex Use the prepare callback to retrieve all track indexes.
         */
        virtual void selectTrack(int trackIndex) = 0;

        /**
         @brief 设置转换播放的url的回调函数，一般用于p2p中的url地址转换
         @param callback 回调函数指针
         */
        /****
         @brief Set a URL conversion callback. URL conversion is typically used in P2P broadcasting.
         @param callback The function pointer of the callback.
         */
        virtual void setPlayUrlConvertCallback(PlayURLConvertCallback callback) = 0;

        /**
         @brief 设置回调监听
         */
        /****
         @brief Set callback listener
         */
        virtual void setListener(IAVDListener *listener) = 0;

        /**
         @brief 获取回调监听
         */
        /****
         @brief Query callback listener
         */
        virtual IAVDListener *getListener() = 0;

        /**
         @brief 获取下载之后的文件路径
         */
        /****
         @brief Query the path of the downloaded file.
         */
        virtual const char *getDownloadedFilePath() = 0;
    };

};// namespace alivc_player

#endif /* AliMediaDownloader_h */
