﻿#ifndef APSARAVIDEO_PLAYER_H
#define APSARAVIDEO_PLAYER_H

#include <stdint.h>
#include <stdlib.h>
#include <windows.h>

#include "AVPConfig.h"
#include "AVPDef.h"
#include "AVPListener.h"
#include "AVPMediaInfo.h"
#include "AVPSource.h"
#include "AVPFrameInfo.h"

namespace alivc_player {

    typedef void ALIVC_EXTERN (*ApsaraPlayerLogCallback)(void *userData, int prio, char *buf);

    /**
     deprecated, see RenderingFrameCallback
     @brief 音频渲染回调函数, 返回true则播放器内部不再渲染该帧数据，返回false则渲染。需要在prepare()之前设置
     @param pts 单位微秒
     @param format 参考AVSampleFormat
     */
    /****
     @brief Audio rendering callback function. Return false if you want to the player render this frame, otherwise true.
     @param pts Unit: microsecond
     @param format see AVSampleFormat
     */
    typedef bool ALIVC_EXTERN (*AudioRenderingCallback)(void *userData, int64_t pts, void *pcmData, int sampleRate, int nbSamples,
                                                        int channels, int format);

    /**
     deprecated, see RenderingFrameCallback
     @brief 视频渲染回调函数, 返回true则播放器内部不再渲染该帧数据，返回false则渲染。需要在prepare()之前设置
     @param pts 单位微秒
     @param dropped 该帧被丢弃，不会被渲染
     */
    /****
     @brief Video rendering callback function. Return false if you want to the player render this frame, otherwise true.
     @param pts Unit: microsecond
     */
    typedef bool ALIVC_EXTERN (*VideoRenderingCallback)(void *userData, int64_t pts, bool dropped, void **data, int *lineSize, int width,
                                                        int height, int rotation, AVPPixelFormat format);

    /**
     @brief 音视频渲染回调函数, 返回true则播放器内部不再渲染该帧数据，返回false则渲染。需要在prepare()之前设置
     @param frameInfo 音频或视频帧数据
     */
    /****
     @brief Audio/Video rendering callback function. Return false if you want to the player render this frame, otherwise true.
     @param frameInfo audio or video frame info
     */
    typedef bool ALIVC_EXTERN (*RenderingFrameCallback)(void *userData, AVPFrameInfo* frameInfo);

    typedef void ALIVC_EXTERN (*EventReportCallback)(void *userData, const char* jsonParams);

    void ALIVC_EXTERN InitPrivateService(const char *fileContentBuffer, int bufferLength);
    void ALIVC_EXTERN InitPrivateService(const char *filePath);

    class ALIVC_EXTERN AliPlayer {
    public:
        static AliPlayer *CreatePlayer();
        static void DestroyPlayer(AliPlayer *player);

        virtual ~AliPlayer() = default;

        /**
         @brief 使用url方式来播放视频
         @param source AVPUrlSource的输入类型
         @see AVPUrlSource
         */
        /****
         @brief Play by URL.
         @param source AVPUrlSource type.
         @see AVPUrlSource
         */
        virtual void setSource(const AVPUrlSource &source) = 0;

        /**
         @brief 用vid和sts来播放视频，sts可参考：https://help.aliyun.com/document_detail/28756.html
         @param source AVPVidStsSource的输入类型
         @see AVPVidStsSource
         */
        /****
         @brief Play by VID and STS. For STS, see https://help.aliyun.com/document_detail/28756.html
         @param source AVPVidStsSource type.
         @see AVPVidStsSource
         */
        virtual void setSource(const AVPVidStsSource &source) = 0;

        /**
         @brief 用vid和MPS信息来播放视频。可参考：https://help.aliyun.com/document_detail/53522.html
         @param source AVPVidMpsSource的输入类型
         @see AVPVidMpsSource
         */
        /****
         @brief Play by VID and MPS. See https://help.aliyun.com/document_detail/53522.html
         @param source AVPVidMpsSource type.
         @see AVPVidMpsSource
         */
        virtual void setSource(const AVPVidMpsSource &source) = 0;

        /**
         @brief 使用vid+playauth方式播放。可参考：https://help.aliyun.com/document_detail/57294.html
         @param source AVPVidAuthSource的输入类型
         @see AVPVidAuthSource
         */
        /****
         @brief Play by VID and playauth. See https://help.aliyun.com/document_detail/57294.html
         @param source AVPVidAuthSource type.
         @see AVPVidAuthSource
         */
        virtual void setSource(const AVPVidAuthSource &source) = 0;

        /**
         @brief 播放准备，异步
         */
        /****
         @brief Prepare for playback. Asynchronous callback.
         */
        virtual void prepare() = 0;

        /**
         @brief 开始播放
         */
        /****
         @brief Start playing.
         */
        virtual void start() = 0;

        /**
         @brief 暂停播放
         */
        /****
         @brief Pause.
         */
        virtual void pause() = 0;

        /**
         @brief 重置播放
         */
        /****
         @brief Reset.
         */
        virtual void reset() = 0;

        /**
         @brief 停止播放
         */
        /****
         @brief Stop.
         */
        virtual void stop() = 0;

        /**
         @brief 刷新view，例如view size变化时。
         */
        /****
         @brief Refresh the view in certain situations, for example, when the size of the view changes.
         */
        virtual void redraw() = 0;

        /**
         @brief 截图, 数据在onSnapshotImageBuffer回调中返回
         */
        /****
         @brief Snapshot. data will be returned in onSnapshotImageBuffer callback
         */
        virtual void snapshot() = 0;

        /**
         @brief 设置监听
         */
        /****
         @brief set listener
         */
        virtual void setListener(IAVPListener *listener) = 0;

        /**
         @brief 获取监听
         */
        /****
         @brief get listener
         */
        virtual IAVPListener *getListener() = 0;

        /**
         @brief 获取主时钟pts
         */
        /****
         @brief get master clock pts
         */
        virtual int64_t getMasterClockPts() = 0;

        /**
         @brief 跳转到指定的播放位置
         @param time 新的播放位置，单位毫秒
         @param seekMode seek模式
         @see AVPSeekMode
         */
        /****
         @brief Seek to a specified position.
         @param time The specified position that the player will seek to, Unit: millisecond.
         @param seekMode Seek mode.
         @see AVPSeekMode
         */
        virtual void seekToTime(int64_t time, AVPSeekMode seekMode) = 0;

        /**
         @brief 根据trackIndex，切换清晰度
         @param trackIndex 选择清晰度的index，SELECT_AVPTRACK_TYPE_VIDEO_AUTO代表自适应码率
         */
        /****
         @brief Switch definition according to the specified track index.
         @param trackIndex Specify a definition index. SELECT_AVPTRACK_TYPE_VIDEO_AUTO: auto bitrate adjustment.
         */
        virtual void selectTrack(int trackIndex) = 0;

        /**
         @brief 获取媒体信息，包括track信息
         */
        /****
         @brief Query media information, including track information.
         */
        virtual AVPMediaInfo getMediaInfo() = 0;

        /**
         @brief 获取当前播放track
         @param type track类型
         @see AVPTrackType
         */
        /****
         @brief Query the track that is being played.
         @param type Track type.
         @see AVPTrackType
         */
        virtual AVPTrackInfo getCurrentTrack(AVPTrackType type) = 0;

        /**
         @brief 获取指定位置的缩略图
         @param positionMs 代表在哪个指定位置的缩略图
         */
        /****
         @brief Retrieve the thumbnail image at the specified position.
         @param positionMs The specified position.
         */
        virtual void getThumbnail(int64_t positionMs) = 0;

        /**
         @brief 用于跟踪日志，设为"DisableAnalytics"可关闭日志分析系统（不推荐）。
         @param traceID 指定和其他客户端连接可跟踪的id
         */
        /****
         @brief Set a trace ID for debugging. Set as "DisableAnalytics" to disable report analytics data to server(not recommended).
         @param traceID The specified client connection trace ID.
         */
        virtual void setTraceID(const char *traceID) = 0;

        /**
         @brief 播放器设置
         @param config AVPConfig类型
         @see AVPConfig
         */
        /****
         @brief Modify player configuration.
         @param config AVPConfig type.
         @see AVPConfig
         */
        virtual void setConfig(AVPConfig *config) = 0;

        /**
         @brief 获取播放器设置
         @see AVPConfig
         */
        /****
         @brief Query player configuration.
         @see AVPConfig
         */
        virtual AVPConfig *getConfig() = 0;

        /**
         @brief 设置缓存配置
         @param AVPCacheConfig 缓存配置。{@link AVPCacheConfig}。
         @return 设置成功返回YES
         */
        /****
         @brief Modify cache configuration.
         @param AVPCacheConfig Cache configuration. {@link AVPCacheConfig}.
         @return If the cache configuration is modified, YES is returned.
         */
        virtual bool setCacheConfig(AVPCacheConfig *cacheConfig) = 0;

        /**
         @brief 根据url获取缓存的文件名。
         @brief 如果没有实现delegate，将会采用默认逻辑生成文件名。
         @param URL URL
         @return 最终缓存的文件名（不包括后缀。缓存文件的后缀名统一为.alv）。使用后需要使用alivcFree释放。
         */
        /****
         @brief Query the name of the cached file with the specified URL. 
         @brief If the delegation failed, the default logic is used to generate the file name.
         @param URL The URL of the cached file.
         @return The name of the cached file. The file extension is not included. All files use the .alv extension. should be freed width alivcFree()
         */
        virtual const char *getCacheFilePath(const char *URL) = 0;


        /**
         @brief 根据vid获取缓存的文件名。如果有自定义的规则。
         @param vid        视频id
         @param format     视频格式
         @param definition 视频清晰度
         @param previewTime 试看时长, 0则忽略该参数
         @return 最终缓存的文件名（不包括后缀。缓存文件的后缀名统一为.alv）。使用后需要使用alivcFree释放。
         */
        /****
         @brief Query the name of the cached file with the specified URL. 
         @param vid        The VID of the video.
         @param format     The format of the video.
         @param definition The definition of the video.
         @param previewTime The preview duration of the video. if 0, ignore this param
         @return The name of the cached file. The file extension is not included. All files use the .alv extension.should be freed width alivcFree()
         */
        virtual const char *getCacheFilePath(const char *vid, const char *format, const char *definition, int previewTime) = 0;

        /**
         @brief 释放内部分配的内存
         @see getCacheFilePath
         */
        /****
         @brief free memory allocate by sdk
         @see getCacheFilePath
         */
        static void alivcFree(void *buffer);

        /**
         @brief 添加外挂字幕。
         @param URL 字幕地址
         */
        /****
         @brief Add external subtitles
         @param URL subtitle address
         */
        virtual void addExtSubtitle(const char *URL) = 0;

        /**
         @brief 选择外挂字幕
         @param trackIndex 字幕索引
         @param enable true：选择，false：关闭
         */
        /****
         @brief Select external subtitles
         @param trackIndex caption index
         @param enable true: select, false: close
         */
        virtual void selectExtSubtitle(int trackIndex, bool enable) = 0;

        /**
         @brief 获取SDK版本号信息
         */
        /****
         @brief Query the SDK version.
         */
        static const char *getSDKVersion();

        /**
         @brief 设置是否静音
         */
        /****
         @brief Mute or unmute the player
         */
        virtual void setMute(bool muted) = 0;

        /**
         @brief 获取是否静音
         */
        /****
         @brief Get mute status
         */
        virtual bool isMuted() = 0;

        /**
         @brief 播放速率，0.5-2.0之间，1为正常播放
         */
        /****
         @brief Set the playback speed. Valid values: 0.5 to 2.0. Value 1 indicates normal speed
         */
        virtual void setRate(float rate) = 0;

        /**
         @brief 获取播放速率
         */
        /****
         @brief Query the playback speed
         */
        virtual float getRate() = 0;

        /**
         @brief 是否开启硬件解码
         */
        /****
         @brief Enable or disable hardware decoding
         */
        virtual void enableHardwareDecoder(bool bEnabled) = 0;

        /**
         @brief 获取是否开启硬件解码
         */
        /****
         @brief Query whether enable hardware decoding
         */
        virtual bool isEnabledHardwareDecoder() = 0;

        /**
         @brief 设置是否循环播放
         */
        /****
         @brief Enable or disable loop playback
         */
        virtual void setLoop(bool bLoop) = 0;

        /**
         @brief 获取是否循环播放
         */
        /****
         @brief Query whether enable loop playback
         */
        virtual bool isLoop() = 0;

        /**
         @brief 设置是否自动播放
         */
        /****
         @brief Enable or disable autoplay
         */
        virtual void setAutoPlay(bool bAutoPlay) = 0;

        /**
         @brief 获取是否自动播放
         */
        /****
         @brief Query whether enableautoplay
         */
        virtual bool isAutoPlay() = 0;

        /**
         @brief 渲染镜像模式
         @see AVPMirrorMode
         */
        /****
         @brief Set a mirroring mode
         @see AVPMirrorMode
         */
        virtual void setMirrorMode(AVPMirrorMode mirrorMode) = 0;

        /**
         @brief 获取渲染镜像模式
         @see AVPMirrorMode
         */
        /****
         @brief Query mirroring mode
         @see AVPMirrorMode
         */
        virtual AVPMirrorMode getMirrorMode() = 0;

        /**
         @brief 渲染旋转模式
         @see AVPRotateMode
         */
        /****
         @brief Set a rotate mode
         @see AVPRotateMode
         */
        virtual void setRotateMode(AVPRotateMode rotateMode) = 0;

        /**
         @brief 获取渲染旋转模式
         @see AVPRotateMode
         */
        /****
         @brief Query rotate mode
         @see AVPRotateMode
         */
        virtual AVPRotateMode getRotateMode() = 0;

        /**
         @brief 渲染填充模式
         @see AVPScalingMode
         */
        /****
         @brief Set a zoom mode
         @see AVPScalingMode
         */
        virtual void setScalingMode(AVPScalingMode scalingMode) = 0;

        /**
         @brief 获取渲染填充模式
         @see AVPScalingMode
         */
        /****
         @brief Query zoom mode
         @see AVPScalingMode
         */
        virtual AVPScalingMode getScalingMode() = 0;

        /**
         @brief 设置播放器的视图playerView
         * view: windows下为HWND
         */
        /****
         @brief Set a player view (playerView).
         * view: windows platform: HWND.
         */
        virtual void setView(void *view) = 0;

        /**
         @brief 获取播放器的视图view
         */
        /**
         @brief Query player view
         */
        virtual void *getView() = 0;

        /**
         @brief 获取视频的宽度
         */
        /****
         @brief Query the width of the video
         */
        virtual int getWidth() = 0;

        /**
         @brief 获取视频的高度
         */
        /****
         @brief Query the height of the video
         */
        virtual int getHeight() = 0;

        /**
         @brief 获取视频的旋转角度，从metadata中获取出来
         */
        /****
         @brief Query the rotate angle of the video, which is retrieved from the video metadata
         */
        virtual int getRotation() = 0;

        /**
         @brief 设置播放器的音量（非系统音量），范围0.0~2.0，当音量大于1.0时，可能出现噪音，不推荐使用。
         */
        /****
         @brief Set the volume of the player(Not system volume). The range is 0.0~2.0，it maybe lead to noise if set volume more then 1.0, not recommended.
         */
        virtual void setVolume(float volume) = 0;

        /**
         @brief 获取播放器的音量（非系统音量）
         */
        /****
         @brief Query the volume of the player(Not system volume).
         */
        virtual float getVolume() = 0;

        /**
         @brief 获取视频的长度
         */
        /****
         @brief Query the length of the video
         */
        virtual int64_t getDuration() = 0;

        /**
         @brief 获取当前播放位置
         */
        /****
         @brief Query the current playback position
         */
        virtual int64_t getCurrentPosition() = 0;

        /**
         @brief 获取已经缓存的位置
         */
        /****
         @brief Query the buffered position
         */
        virtual int64_t getBufferedPosition() = 0;

        /**
         @brief 设置log输出级别
         */
        /****
         @brief set the level of the log.
         */
        static void enableLog(AVPLogLevel level);

        /**
         @brief 设置日志打印回调，异步
         */
        /****
         @brief Set a log output callback block. Asynchronous.
         */
        static void setLogCallback(ApsaraPlayerLogCallback func, void *arg);

        /**
         deprecated, see setRenderingFrameCallback
         @brief 设置音频渲染回调。需要在prepare()之前设置
         */
        /****
         @brief Set a audio rendering callback block. 
         */
        virtual void setAudioRenderingCallback(AudioRenderingCallback func, void *userData) = 0;

        /**
         deprecated, see setRenderingFrameCallback
         @brief 设置视频渲染回调。需要在prepare()之前设置
         */
        /****
         @brief Set a video rendering callback block. 
         */
        virtual void setVideoRenderingCallback(VideoRenderingCallback func, void *userData) = 0;

        /**
         @brief 设置音视频视频渲染回调。需要在prepare()之前设置
         */
        /****
         @brief Set a audio/video rendering callback block. 
         */
        virtual void setRenderingFrameCallback(RenderingFrameCallback func, void *userData) = 0;

        /**
         @brief 设置事件上报回调。
         */
        /****
         @brief Set a event report callback block. 
         */
        virtual void setEventReprotCallback(EventReportCallback func, void *userData) = 0;
    };
};    // namespace alivc_player
#endif// APSARAVIDEO_PLAYER_H
